/************************************************************\
 * Copyright (C) 2024 The Regents of the University of California.
 * (c.f. DISCLAIMER, COPYING)
 *
 * This file is part of PowerMan, a remote power management program.
 * For details, see https://github.com/chaos/powerman.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
\************************************************************/

#ifndef REDFISHPOWER_DEFS_H
#define REDFISHPOWER_DEFS_H

#define CMD_STAT       "stat"
#define CMD_ON         "on"
#define CMD_OFF        "off"

#endif /* REDFISHPOWER_DEFS_H */

/*
 * vi:tabstop=4 shiftwidth=4 expandtab
 */
